@@ -8,11 +8,10 @@ import (
 	"fmt"
 
 	"github.com/cilium/ebpf"
-	"github.com/cilium/ebpf/btf"
 	"golang.org/x/sys/unix"
 )
 
-type BpfProgName2Addr map[string]uint64
+var errNotFound = errors.New("not found")
 
 func listBpfProgs(typ ebpf.ProgramType) ([]*ebpf.Program, error) {
 	var (
@@ -41,34 +40,35 @@ func listBpfProgs(typ ebpf.ProgramType) ([]*ebpf.Program, error) {
 	return progs, nil
 }
 
-func getEntryFuncName(prog *ebpf.Program) (string, string, error) {
+func getBpfProgInfo(prog *ebpf.Program) (entryFuncName string, err error) {
 	info, err := prog.Info()
 	if err != nil {
-		return "", "", fmt.Errorf("failed to get program info: %w", err)
+		err = fmt.Errorf("failed to get program info: %w", err)
+		return
 	}
 
-	id, ok := info.BTFID()
+	_, ok := info.BTFID()
 	if !ok {
-		return "", "", fmt.Errorf("bpf program %s does not have BTF", info.Name)
+		// FENTRY/FEXIT program can only be attached to another program
+		// annotated with BTF. So if the BTF ID is not found, it means
+		// the program is not annotated with BTF.
+		err = errNotFound
+		return
 	}
 
-	handle, err := btf.NewHandleFromID(id)
+	insns, err := info.Instructions()
 	if err != nil {
-		return "", "", fmt.Errorf("failed to get BTF handle: %w", err)
-	}
-	defer handle.Close()
-
-	spec, err := handle.Spec(nil)
-	if err != nil {
-		return "", "", fmt.Errorf("failed to get BTF spec: %w", err)
+		err = fmt.Errorf("failed to get program instructions: %w", err)
+		return
 	}
 
-	iter := spec.Iterate()
-	for iter.Next() {
-		if fn, ok := iter.Type.(*btf.Func); ok {
-			return fn.Name, info.Name, nil
+	for _, insn := range insns {
+		sym := insn.Symbol()
+		if sym != "" {
+			return sym, nil
 		}
 	}
 
-	return "", "", fmt.Errorf("no function found in %s bpf prog", info.Name)
+	err = errNotFound
+	return
 }
